import importlib

from statuscheck.services import SERVICES


def _import_module(service):
    return importlib.import_module(f"statuscheck.services.{service}")


def get_statuscheck_api(service):
    module = _import_module(service)
    return module.ServiceAPI()


def get_available_services():
    return SERVICES



from typing import NamedTuple

import requests

from statuscheck.services._base import BaseServiceAPI
from statuscheck.status_types import (
    SPIO_COMPONENT_OPERATIONAL,
    SPIO_COMPONENTS_STATUSES,
    SPIO_INCIDENTS_STATUSES,
    SPIO_INDICATORS,
)


class StatusPageIOSummary(NamedTuple):
    name: str
    status: str
    incidents: list
    components: list

    @classmethod
    def _get_components(cls, data):
        all_components = data["components"]
        damaged_components = [
            c for c in all_components if c["status"] != SPIO_COMPONENT_OPERATIONAL
        ]
        filtered_data = []
        important_keys = ("name", "status", "description")
        for component in damaged_components:
            component_data = {k: component.get(k, None) for k in important_keys}
            component_data["status"] = SPIO_COMPONENTS_STATUSES[
                component_data["status"]
            ]
            filtered_data.append(component_data)
        return filtered_data

    @classmethod
    def _get_incidents(cls, data):
        incidents = data["incidents"]
        filtered_data = []
        important_keys = ("name", "status", "impact")
        for component in incidents:
            component_data = {k: component.get(k, None) for k in important_keys}
            component_data["status"] = SPIO_INCIDENTS_STATUSES[component_data["status"]]
            filtered_data.append(component_data)
        return filtered_data

    @classmethod
    def from_data(cls, name, data):
        status = data["status"]
        status_type = SPIO_INDICATORS[status["indicator"]]
        return cls(
            name=name,
            status=status_type,
            incidents=cls._get_incidents(data),
            components=cls._get_components(data),
        )


class BaseStatusPageAPI(BaseServiceAPI):
    domain_id: str = None

    def _get_base_url(self):
        """Statuspage.io API URL for given service."""
        if not self.domain_id:
            raise NotImplementedError("Please, add domain id")
        return f"https://{self.domain_id}.statuspage.io/api/v2/"

    def get_summary(self):
        url = self._get_base_url() + "summary.json"
        response = requests.get(url)
        response.raise_for_status()
        return StatusPageIOSummary.from_data(name=self.name, data=response.json())



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Travis CI"
    domain_id = "pnpcptp8xh9k"
    status_url = "https://www.traviscistatus.com/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Personio"
    domain_id = "68kf6yfjnk2d"
    status_url = "https://status.personio.de/"



import os

import pytest

from statuscheck.check import get_available_services, get_statuscheck_api
from statuscheck.services import SERVICES

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def test_get_available_services():
    services = get_available_services()
    assert services

    service_files = os.listdir(os.path.join(BASE_DIR, "statuscheck", "services"))
    excluded_files = (
        "__pycache__",
        "__init__.py",
        "_base.py",
        "_custompage.py",
        "_statuspage.py",
        "_statusio.py",
    )
    service_files = [f[:-3] for f in service_files if f not in excluded_files]
    assert set(services) == set(service_files)


@pytest.mark.parametrize("service", SERVICES)
def test_get_statuscheck_api(service):
    api = get_statuscheck_api(service)
    assert api._module_name == service
    assert api.status_url
    assert api.summary
    print(api.summary)



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "PagerDuty"
    domain_id = "33yy6hwxnwr3"
    status_url = "https://status.pagerduty.com/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Atlassian"
    domain_id = "x67gp49yvrzv"
    status_url = "https://status.atlassian.com/"



from statuscheck.services._statusio import BaseStatusIOAPI


class ServiceAPI(BaseStatusIOAPI):
    name = "GitLab"
    domain_id = "5b36dc6502d06804c08349f7"
    status_url = "https://status.gitlab.com/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "SparkPost"
    domain_id = "7ky1q6zd3fyp"
    status_url = "https://status.sparkpost.com/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Codecov"
    domain_id = "wdzsn5dlywj9"
    status_url = "https://status.codecov.io/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Bitbucket"
    domain_id = "bqlf8qjztdtr"
    status_url = "https://status.bitbucket.org/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "CircleCI"
    domain_id = "6w4r0ttlx5ft"
    status_url = "https://circleci.statuspage.io/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "DigitalOcean"
    domain_id = "s2k7tnzlhrpw"
    status_url = "https://status.digitalocean.com/"



from typing import NamedTuple

import requests

import statuscheck.status_types
from statuscheck.services._base import BaseServiceAPI


class SalesforceSummary(NamedTuple):
    name: str
    status: str
    incidents: list
    components: list

    @classmethod
    def _get_incidents(cls, summary):
        pass

    @classmethod
    def from_data(cls, name, data):
        return cls(
            name=name,
            status=data["status"],
            incidents=data["incidents"],
            components=data["components"],
        )


class ServiceAPI(BaseServiceAPI):
    STATUS_OK = "OK"
    STATUS_UNKNOWN = "UNKNOWN"
    STATUS_CORE_OUTAGE = "MAJOR_INCIDENT_CORE"
    STATUS_NONCORE_OUTAGE = "MAJOR_INCIDENT_NONCORE"
    STATUS_CORE_INCIDENT = "MINOR_INCIDENT_CORE"
    STATUS_NONCORE_INCIDENT = "MINOR_INCIDENT_NONCORE"
    STATUS_CORE_MAINTENANCE = "MAINTENANCE_CORE"
    STATUS_NONCORE_MAINTENANCE = "MAINTENANCE_NONCORE"

    # sorted by severity
    NOT_GOOD_STATUSES = [
        STATUS_CORE_OUTAGE,
        STATUS_NONCORE_OUTAGE,
        STATUS_CORE_INCIDENT,
        STATUS_NONCORE_INCIDENT,
        STATUS_CORE_MAINTENANCE,
        STATUS_NONCORE_MAINTENANCE,
    ]

    STATUS_TYPE_MAPPING = {
        STATUS_OK: statuscheck.status_types.TYPE_GOOD,
        STATUS_CORE_INCIDENT: statuscheck.status_types.TYPE_INCIDENT,
        STATUS_NONCORE_INCIDENT: statuscheck.status_types.TYPE_INCIDENT,
        STATUS_CORE_OUTAGE: statuscheck.status_types.TYPE_OUTAGE,
        STATUS_NONCORE_OUTAGE: statuscheck.status_types.TYPE_OUTAGE,
        STATUS_CORE_MAINTENANCE: statuscheck.status_types.TYPE_MAINTENANCE,
        STATUS_NONCORE_MAINTENANCE: statuscheck.status_types.TYPE_MAINTENANCE,
        STATUS_UNKNOWN: statuscheck.status_types.TYPE_UNKNOWN,
    }

    name = "Salesforce"
    base_url = "https://api.status.salesforce.com/v1/"
    status_url = "https://status.salesforce.com"

    def get_summary(self):
        self.data = self._get_status_data()
        return SalesforceSummary.from_data(
            name=self.name,
            data={
                "status": self.get_general_status(),
                "incidents": self.get_incidents(),
                "components": self.get_components(),
            },
        )

    def _get_status_data(self):
        url = self.base_url + "instances/status/preview"
        response = requests.get(url)
        response.raise_for_status()
        return response.json()

    def get_components(self):
        servers = self._get_affected_servers()
        for server in servers:
            server["status"] = self.STATUS_TYPE_MAPPING[server["status"]]
        return servers

    def _get_affected_servers(self) -> list:
        servers = []
        for server in self.data:
            if server.get("status") != self.STATUS_OK:
                server["name"] = server["key"]
                servers.append(server)
        return servers

    def _get_general_incidents(self):
        incidents = self.get_incidents()
        general_incidents = []
        for incident in incidents:
            if incident["affectsAll"]:
                general_incidents.append(incident)
        return general_incidents

    def get_incidents(self):
        incident_keys = []
        incidents = []
        affected_servers = self._get_affected_servers()
        affected_server_keys = [s["key"] for s in affected_servers]
        if not affected_servers:
            return []

        for server in affected_servers:
            if server["key"] not in affected_server_keys:
                continue

            server_incidents = server["Incidents"]
            for incident in server_incidents:
                if incident["id"] in incident_keys:
                    continue

                incident["name"] = incident["IncidentImpacts"][0]["type"]
                incident["server_status"] = self.STATUS_TYPE_MAPPING[server["status"]]
                incidents.append(incident)
                incident_keys.append(incident["id"])

        return incidents

    def get_general_status(self):
        incidents = self._get_general_incidents()
        if not incidents:
            return self.STATUS_TYPE_MAPPING[self.STATUS_OK]

        return incidents[0]["server_status"]



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Redis Labs"
    domain_id = "j6ydyp75k0zx"
    status_url = "https://status.redislabs.com/"



import sys

import click

from statuscheck.check import get_statuscheck_api
from statuscheck.services import SERVICES

from .__about__ import __url__


@click.command()
@click.argument("service")
def main(service):
    """Console script for statuscheck."""
    if service == "all":
        _check_all()
        return 0

    try:
        service_api = get_statuscheck_api(service)
    except ModuleNotFoundError:
        click.echo(f'"{service}" is not implemented, leave a note at {__url__}')
        return 1

    _report_status(service_api)

    return 0


def _report_status(service_api):
    summary = service_api.get_summary()

    click.echo(f"Current status of {service_api.name}: {summary.status}")

    incidents = summary.incidents
    if incidents:
        click.echo(f"Registered incidents:")
        for incident in incidents:
            incident_name = incident["name"]
            incident_status = incident.get("status")
            if incident_status:
                click.echo(f"- {incident_name} [{incident_status}]")
            else:
                click.echo(f"- {incident_name}")

    has_components = hasattr(summary, "components")
    if has_components and summary.components:
        click.echo(f"Affected components:")
        for component in summary.components:
            name = component["name"]
            component_status = component["status"]
            click.echo(f"- {name}: {component_status}")

    if incidents or (has_components and summary.components):
        click.echo()
        click.echo(f"More: {service_api.status_url}")


def _check_all():
    for service in SERVICES:
        service_api = get_statuscheck_api(service)
        _report_status(service_api=service_api)
        click.echo("=" * 40)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



from typing import NamedTuple

import requests

import statuscheck.status_types
from statuscheck.services._base import BaseServiceAPI

STATUS_TYPE_MAPPING = {
    "green": statuscheck.status_types.TYPE_GOOD,
    "yellow": statuscheck.status_types.TYPE_INCIDENT,
    "red": statuscheck.status_types.TYPE_OUTAGE,
    "blue": statuscheck.status_types.TYPE_MAINTENANCE,
}


class ServiceSummary(NamedTuple):
    name: str
    status: str
    status_development: str
    status_production: str
    incidents: list

    @classmethod
    def _get_incidents(cls, summary):
        incidents = summary["issues"]
        filtered_data = []
        important_keys = ("title", "status_dev", "status_prod", "full_url")
        for component in incidents:
            component_data = {k: component.get(k, None) for k in important_keys}
            component_data["name"] = component_data.pop("title")
            filtered_data.append(component_data)
        return filtered_data

    @classmethod
    def from_data(cls, name, data):
        status_data = data["status"]
        status_production = STATUS_TYPE_MAPPING[status_data["Production"]]
        status_development = STATUS_TYPE_MAPPING[status_data["Development"]]
        status = status_production

        if status_development != statuscheck.status_types.TYPE_GOOD:
            status = status_development
        if status_production != statuscheck.status_types.TYPE_GOOD:
            status = status_production

        return cls(
            name=name,
            status=status,
            status_production=status_production,
            status_development=status_development,
            incidents=cls._get_incidents(data),
        )


class ServiceAPI(BaseServiceAPI):
    name = "Heroku"
    base_url = "https://status.heroku.com/api/v3/"
    status_url = "https://status.heroku.com"

    def get_summary(self):
        url = self.base_url + "current-status"
        response = requests.get(url)
        response.raise_for_status()
        return ServiceSummary.from_data(name=self.name, data=response.json())



from typing import NamedTuple

import statuscheck.status_types
from statuscheck.services._custompage import BaseCustomStatusPageAPI


class SlackSummary(NamedTuple):
    name: str
    status: str
    incidents: list

    @classmethod
    def from_data(cls, name, data):
        return cls(name=name, status=data["status"], incidents=data["incidents"])


class ServiceAPI(BaseCustomStatusPageAPI):
    STATUS_TYPE_MAPPING = {
        "Ok": statuscheck.status_types.TYPE_GOOD,
        "Incident": statuscheck.status_types.TYPE_INCIDENT,
        "Outage": statuscheck.status_types.TYPE_OUTAGE,
        "Maintenance": statuscheck.status_types.TYPE_MAINTENANCE,
    }

    name = "Slack"
    base_url = "https://status.slack.com/"
    status_url = base_url

    def get_summary(self):
        html = self._get_html_response()
        html_container = html.find(".container", first=True)

        status_icon = html_container.find("img", first=True).attrs.get("src")
        status_icon_type = status_icon.rsplit(".")[0].split("/")[-1]
        status = self.STATUS_TYPE_MAPPING.get(status_icon_type, "")
        description = html_container.find("h1", first=True).text

        # TODO: parse Slack components and their statuses
        incidents = []
        if status != statuscheck.status_types.TYPE_GOOD:
            incidents.append({"name": description, "status": status})
        return SlackSummary.from_data(
            name=self.name, data={"status": status, "incidents": incidents}
        )



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "GitHub"
    domain_id = "kctbh9vrtdwd"
    status_url = "https://www.githubstatus.com/"



from typing import NamedTuple

import requests

from statuscheck.services._base import BaseServiceAPI
from statuscheck.status_types import (
    TYPE_GOOD,
    TYPE_INCIDENT,
    TYPE_OUTAGE,
    TYPE_SECURITY,
)

STATUS_TYPE_MAPPING = {
    "Operational": TYPE_GOOD,
    "Degraded Performance": TYPE_INCIDENT,
    "Full Service Disruption": TYPE_OUTAGE,
    "Security Issue": TYPE_SECURITY,
}


class StatusIOSummary(NamedTuple):
    name: str
    status: str
    incidents: list
    components: list

    @classmethod
    def _get_components(cls, data):
        all_components = data["status"]
        damaged_components = [c for c in all_components if c["status"] != "Operational"]
        filtered_data = []
        important_keys = ("name", "status")
        for component in damaged_components:
            filtered_data.append({k: component.get(k, None) for k in important_keys})
        return filtered_data

    @classmethod
    def _get_incidents(cls, data):
        return data["incidents"]

    @classmethod
    def from_data(cls, name, data):
        status_raw = data["status_overall"]["status"]
        status = STATUS_TYPE_MAPPING.get(status_raw, "")
        return cls(
            name=name,
            status=status,
            incidents=cls._get_incidents(data),
            components=cls._get_components(data),
        )


class BaseStatusIOAPI(BaseServiceAPI):
    """
    Status.io pages API handler.

    API v2: https://statusio.docs.apiary.io
    Public status API: https://kb.status.io/developers/public-status-api/
    """

    domain_id: str = None

    def _get_base_url(self):
        if not self.domain_id:
            raise NotImplementedError("Please, add domain key")
        return f"https://api.status.io/1.0/status/{self.domain_id}"

    def get_summary(self):
        response = requests.get(self._get_base_url())
        response.raise_for_status()
        return StatusIOSummary.from_data(name=self.name, data=response.json()["result"])



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "CloudAMQP"
    domain_id = "2126xm8ctzlk"
    status_url = "https://status.cloudamqp.com/"



SERVICES = [
    "asana",
    "atlassian",
    "bitbucket",
    "circleci",
    "close",
    "cloudamqp",
    "cloudflare",
    "codecov",
    "digitalocean",
    "discord",
    "docker",
    "github",
    "gitlab",
    "invision",
    "heroku",
    "mailgun",
    "npm",
    "nylas",
    "pagerduty",
    "personio",
    "pypi",
    "redislabs",
    "salesforce",
    "sentry",
    "servicemax",
    "signal",
    "slack",
    "sparkpost",
    "travis",
    "twilio",
]

__all__ = [SERVICES]



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Discord"
    domain_id = "srhpyqt94yxb"
    status_url = "https://status.discordapp.com/"



from click.testing import CliRunner

from statuscheck import cli


def test_cli_no_args():
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 2


def test_cli():
    runner = CliRunner()
    result = runner.invoke(cli.main, "github")
    assert result.exit_code == 0


def test_cli_all():
    runner = CliRunner()
    result = runner.invoke(cli.main, "all")
    assert result.exit_code == 0



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "ServiceMax"
    domain_id = "jtvt0b51c5wy"
    status_url = "https://servicemax.statuspage.io/"



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Cloudflare"
    domain_id = "yh6f0r4529hb"
    status_url = "https://www.cloudflarestatus.com/"



import logging

logger = logging.getLogger(__name__)


class BaseServiceAPI:
    name: str = None
    summary = None

    @property
    def _module_name(self):
        module_relpath = self.__class__.__module__
        return module_relpath.rsplit(".", 1)[1]

    def __init__(self):
        self.summary = self.get_summary()

    def get_summary(self):
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

about = {}

with open(os.path.join(here, "statuscheck", "__about__.py")) as f:
    exec(f.read(), about)

requirements = ["Click>=6.0", "requests", "requests_html", "feedparser"]

test_requirements = ["pytest"]

setup(
    author=about["__author__"],
    author_email=about["__email__"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="Tool to check PAAS/SAAS status pages",
    entry_points={"console_scripts": ["statuscheck=statuscheck.cli:main"]},
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="statuscheck",
    name="statuscheck",
    packages=find_packages(exclude=["tests", "tests.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url=about["__url__"],
    version=about["__version__"],
    zip_safe=False,
)



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "Mailgun"
    domain_id = "6jp439mdyy0k"
    status_url = "https://status.mailgun.com/"



from typing import NamedTuple

from statuscheck.services._custompage import BaseCustomStatusPageAPI
from statuscheck.status_types import TYPE_GOOD


class SignalSummary(NamedTuple):
    name: str
    status: str
    incidents: list

    @classmethod
    def from_data(cls, name, data):
        return cls(name=name, status=data["status"], incidents=data["incidents"])


class ServiceAPI(BaseCustomStatusPageAPI):
    STATUS_TYPE_MAPPING = {"Signal is up and running.": TYPE_GOOD}

    name = "Signal"
    base_url = "https://status.signal.org/"
    status_url = base_url

    def get_summary(self):
        html = self._get_html_response()
        status_raw = html.find("div", first=True).text
        status_text = status_raw.split("\n", 1)[1]
        # TODO: parse incidents
        status_type = self.STATUS_TYPE_MAPPING.get(status_text, "")
        incidents = []
        if status_type != TYPE_GOOD:
            incidents.append({"name": status_text, "status": status_type})
        return SignalSummary.from_data(
            name=self.name, data={"status": status_type, "incidents": incidents}
        )



from statuscheck.services._statusio import BaseStatusIOAPI


class ServiceAPI(BaseStatusIOAPI):
    name = "Docker"
    domain_id = "533c6539221ae15e3f000031"
    status_url = "https://status.docker.com/"



from requests_html import HTMLSession

from statuscheck.services._base import BaseServiceAPI


class BaseCustomStatusPageAPI(BaseServiceAPI):
    def _get_html_response(self):
        session = HTMLSession()
        response = session.get(self.base_url)
        response.raise_for_status()
        return response.html



from statuscheck.services._statuspage import BaseStatusPageAPI


class ServiceAPI(BaseStatusPageAPI):
    name = "InVision"
    domain_id = "tv3rvsrmny3p"
    status_url = "https://status.invisionapp.com/"



TYPE_GOOD = "No issues"
TYPE_MAINTENANCE = "Maintenance"
TYPE_INCIDENT = "Minor incident"
TYPE_SECURITY = "Security incident"
TYPE_OUTAGE = "Major outage"
TYPE_UNKNOWN = ""

STATUS_TYPES = (
    TYPE_GOOD,
    TYPE_MAINTENANCE,
    TYPE_INCIDENT,
    TYPE_OUTAGE,
    TYPE_UNKNOWN,
    TYPE_SECURITY,
)

# Statuspage.io constants
SPIO_INDICATORS = {
    "none": TYPE_GOOD,
    "minor": TYPE_INCIDENT,
    "major": TYPE_OUTAGE,
    "critical": TYPE_OUTAGE,
    "maintenance": TYPE_MAINTENANCE,
}

SPIO_COMPONENT_OPERATIONAL = "operational"
SPIO_COMPONENT_DEGRADED = "degraded_performance"
SPIO_COMPONENT_PARTIAL_OUTAGE = "partial_outage"
SPIO_COMPONENT_MAJOR_OUTAGE = "major_outage"
SPIO_COMPONENT_MAINTENANCE = "under_maintenance"

SPIO_COMPONENTS_STATUSES = {
    SPIO_COMPONENT_OPERATIONAL: "Operational",
    SPIO_COMPONENT_DEGRADED: "Degraded performance",
    SPIO_COMPONENT_PARTIAL_OUTAGE: "Partial outage",
    SPIO_COMPONENT_MAJOR_OUTAGE: "Major outage",
    SPIO_COMPONENT_MAINTENANCE: "Under Maintenance",
}

SPIO_INCIDENT_INVESTIGATING = "investigating"
SPIO_INCIDENT_IDENTIFIED = "identified"
SPIO_INCIDENT_MONITORING = "monitoring"
SPIO_INCIDENT_RESOLVED = "resolved"
SPIO_INCIDENT_POSTMORTEM = "postmortem"

SPIO_INCIDENTS_STATUSES = {
    SPIO_INCIDENT_INVESTIGATING: "Investigating",
    SPIO_INCIDENT_IDENTIFIED: "Identified",
    SPIO_INCIDENT_MONITORING: "Monitoring",
    SPIO_INCIDENT_RESOLVED: "Resolved",
    SPIO_INCIDENT_POSTMORTEM: "Postmortem",
}
